import { Component, Element, Event, Method, Prop, h, Host } from '@stencil/core';
import { hideLazyLoadImages, afterSwipe, lazyLoadContent } from '@deckdeckgo/slide-utils';
import { DeckdeckgoHighlightCodeTerminal, DeckdeckgoHighlightCodeCarbonTheme } from '@deckdeckgo/highlight-code';
export class DeckdeckgoSlideCode {
  constructor() {
    this.anchor = '// DeckDeckGo';
    this.anchorZoom = '// DeckDeckGoZoom';
    this.hideAnchor = true;
    this.language = 'javascript';
    this.terminal = DeckdeckgoHighlightCodeTerminal.CARBON;
    this.theme = DeckdeckgoHighlightCodeCarbonTheme.DRACULA;
  }
  async componentDidLoad() {
    await hideLazyLoadImages(this.el);
    this.slideDidLoad.emit();
    await this.moveSlots();
  }
  moveSlots() {
    return new Promise((resolve) => {
      const code = this.el.querySelector("[slot='code']");
      const codeComponent = this.el.shadowRoot.querySelector('deckgo-highlight-code');
      if (codeComponent && code) {
        codeComponent.appendChild(code);
      }
      resolve();
    });
  }
  beforeSwipe(_enter, _reveal) {
    return new Promise(async (resolve) => {
      const couldSwipe = await this.scrollToNext(_enter);
      if (couldSwipe) {
        await this.zoomCode(false);
      }
      resolve(couldSwipe);
    });
  }
  afterSwipe() {
    return afterSwipe();
  }
  revealContent() {
    return Promise.resolve();
  }
  hideContent() {
    return Promise.resolve();
  }
  scrollToNext(enter) {
    const element = this.el.shadowRoot.querySelector('deckgo-highlight-code');
    if (element && 'findNextAnchor' in element) {
      return new Promise(async (resolve) => {
        const nextAnchor = await element.findNextAnchor(enter);
        const container = this.el.shadowRoot.querySelector('div.deckgo-slide-code-container');
        if (nextAnchor && container) {
          const previousScrollTop = container.scrollTop;
          container.scrollTop = nextAnchor.offsetTop;
          if ('zoomCode' in element) {
            await element.zoomCode(nextAnchor.hasLineZoom);
          }
          resolve(nextAnchor.offsetTop === 0 && previousScrollTop === 0);
        }
        else {
          resolve(true);
        }
      });
    }
    else {
      return new Promise((resolve) => {
        resolve(true);
      });
    }
  }
  zoomCode(zoom) {
    return new Promise(async (resolve) => {
      const element = this.el.shadowRoot.querySelector('deckgo-highlight-code');
      if (element && 'zoomCode' in element) {
        await element.zoomCode(zoom);
      }
      const title = this.el.querySelector("[slot='title']");
      if (title) {
        title.style.setProperty('opacity', zoom ? '0' : '1');
      }
      resolve();
    });
  }
  lazyLoadContent() {
    return lazyLoadContent(this.el);
  }
  // DeckDeckGo
  render() {
    return (h(Host, { class: { 'deckgo-slide-container': true } },
      h("div", { class: "deckgo-slide" },
        h("slot", { name: "title" }),
        h("div", { class: "deckgo-slide-code-container" },
          h("deckgo-highlight-code", { src: this.src, anchor: this.anchor, anchorZoom: this.anchorZoom, hideAnchor: this.hideAnchor, terminal: this.terminal, theme: this.theme, language: this.language })),
        h("slot", { name: "code" }),
        h("slot", { name: "notes" }),
        h("slot", { name: "actions" }),
        h("slot", { name: "background" }),
        h("slot", { name: "header" }),
        h("slot", { name: "footer" }))));
  }
  static get is() { return "deckgo-slide-code"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["deckdeckgo-slide-code.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["deckdeckgo-slide-code.css"]
  }; }
  static get properties() { return {
    "src": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "src",
      "reflect": false
    },
    "anchor": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "anchor",
      "reflect": false,
      "defaultValue": "'// DeckDeckGo'"
    },
    "anchorZoom": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "anchor-zoom",
      "reflect": false,
      "defaultValue": "'// DeckDeckGoZoom'"
    },
    "hideAnchor": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "hide-anchor",
      "reflect": false,
      "defaultValue": "true"
    },
    "language": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "language",
      "reflect": false,
      "defaultValue": "'javascript'"
    },
    "terminal": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "DeckdeckgoHighlightCodeTerminal",
        "resolved": "DeckdeckgoHighlightCodeTerminal.CARBON | DeckdeckgoHighlightCodeTerminal.NONE | DeckdeckgoHighlightCodeTerminal.UBUNTU",
        "references": {
          "DeckdeckgoHighlightCodeTerminal": {
            "location": "import",
            "path": "@deckdeckgo/highlight-code"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "terminal",
      "reflect": false,
      "defaultValue": "DeckdeckgoHighlightCodeTerminal.CARBON"
    },
    "theme": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "DeckdeckgoHighlightCodeCarbonTheme",
        "resolved": "DeckdeckgoHighlightCodeCarbonTheme",
        "references": {
          "DeckdeckgoHighlightCodeCarbonTheme": {
            "location": "import",
            "path": "@deckdeckgo/highlight-code"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "theme",
      "reflect": false,
      "defaultValue": "DeckdeckgoHighlightCodeCarbonTheme.DRACULA"
    }
  }; }
  static get events() { return [{
      "method": "slideDidLoad",
      "name": "slideDidLoad",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "scrolling",
      "name": "scrolling",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "beforeSwipe": {
      "complexType": {
        "signature": "(_enter: boolean, _reveal: boolean) => Promise<boolean>",
        "parameters": [{
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "afterSwipe": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "revealContent": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "hideContent": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "lazyLoadContent": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
}
